﻿@include inc_commonSAL.js
// sqlLiteWrapper
// (c) 2024 Stephane

// This is an include file script for Directory Opus.
// See https://www.gpsoft.com.au/endpoints/redirect.php?page=scripts for development information.



// Called by Directory Opus to initialize the include file script
function OnInitIncludeFile(initData)
{
	initData.name = "sqlLiteWrapper";
	initData.version = "1.0";
	initData.copyright = "(c) 2024 Stephane";
//	initData.url = "https://resource.dopus.com/c/buttons-scripts/16";
	initData.desc = "";
	initData.min_version = "13.0";
	initData.shared = true;
}

// Returns an object with 2 properties : result = the commande line result, lines = a Vector of strings
// Each entry of the vector is a string  with the result (column values) of the select (each column is separated by |, so .split(/\|/); can make this an array of columns
function ExecuteSelectSQL(sqlitePath, sqlDbPath, sqlStatement) {
	var fsu 	= DOpus.FSUtil;
	var sql 	= fsu.Resolve(sqlitePath);
	var db 		= fsu.Resolve(sqlDbPath);

	var fso 	= new ActiveXObject("Scripting.FilesystemObject");
	var shell   = new ActiveXObject("WScript.Shell");

	var tmpFileBase = "DO.Temp.";
	var tmpFileExt = ".txt";
	var tmpFileStdout = ExtSystem.GetTmpFileName(tmpFileBase, tmpFileExt, fso).fullname;
	var tmpFileStderr = tmpFileStdout+".err"+tmpFileExt;
	
	var cmdLine = '%comspec% /c "echo ' + sqlStatement + ' | ';
	cmdLine += '"' + sql + '" "' + db + '"';
	cmdLine += ' >"'+tmpFileStdout+'" 2>"'+tmpFileStderr+'""'

	//dout ("Built command line = '" + cmdLine + "'");

	var result = {};
	result.cmd = cmdLine;
	dout("Command line SQL Wrapper = '" + cmdLine + "'");
	result.returncode = shell.Run(cmdLine, 0, true);
	result.stdout = ExtSystem.ReadFile(tmpFileStdout, fso); fso.DeleteFile(tmpFileStdout);
	result.stderr = ExtSystem.ReadFile(tmpFileStderr, fso); fso.DeleteFile(tmpFileStderr);

	if (result.stderr.length > 0) dout ("Some errors occured ...", true);
	//dout("Result errors : " + result.stderr);
	// dout("RAW internal Result : (" + result.stdout.length + ") : '" + result.stdout + "'");
	var lines = (result.stdout).split(/\r?\n/);
	// dout(" #" + lines.length +" results (lines in result)");

	var vResult = { "result" : result, "nbLines" : lines.length, "lines" : DOpus.Create.Vector };
	for (var e = new Enumerator(lines); !e.atEnd(); e.moveNext())
	{
		var l = e.item();
		// var detail = l.split('');
		// for (var i = 0; i < detail.length; i++) {
		// 	dout(detail[i]);
		// }

		if (typeof e.item() == "string") {
			dout (">> type=" + typeof l + " (" + l.length + ") : " + l);
			vResult.lines.push_back(e.item());
		}
	}

	return vResult;
}



function ExecuteSelectSQLJSON(sqlitePath, sqlDbPath, sqlStatement) {
	var fsu 	= DOpus.FSUtil;
	var sql 	= fsu.Resolve(sqlitePath);
	var db 		= fsu.Resolve(sqlDbPath);

	var fso 	= new ActiveXObject("Scripting.FilesystemObject");
	var shell   = new ActiveXObject("WScript.Shell");

	var tmpFileBase = "DO.Temp.";
	var tmpFileExt = ".txt";
	var tmpFileStdout = ExtSystem.GetTmpFileName(tmpFileBase, tmpFileExt, fso).fullname;
	var tmpFileStderr = tmpFileStdout+".err"+tmpFileExt;
	
	var cmdLine = '%comspec% /c "echo ' + sqlStatement + ' | ';
	cmdLine += '"' + sql + '" -json "' + db + '"';
	cmdLine += ' >"'+tmpFileStdout+'" 2>"'+tmpFileStderr+'""'

	//dout ("Built command line = '" + cmdLine + "'");

	var result = {};
	result.cmd = cmdLine;
	// dout("Command line SQL Wrapper = '" + cmdLine + "'");
	result.returncode = shell.Run(cmdLine, 0, true);
	result.stdout = ExtSystem.ReadFile(tmpFileStdout, fso); fso.DeleteFile(tmpFileStdout);
	result.stderr = ExtSystem.ReadFile(tmpFileStderr, fso); fso.DeleteFile(tmpFileStderr);

	if (result.stderr.length > 0) dout ("Some errors occured ...", true);
	//dout("Result errors : " + result.stderr);
	// dout("RAW internal Result : (" + result.stdout.length + ") : '" + result.stdout + "'");

	if (result.stdout.length == 0) {
		dout("No row returned.", true, true);
		return { "result": result, "nbLines": 0, "json" : { } };	
		// return { "result": result, "nbLines": 0, "json" : { "sqlRequestReturn" : "empty" } };	
	} 

	try {
		//dout("out length = " + result.stdout.length);
		var jsObj = JSON.parse(result.stdout);
		var vResult = { "result" : result, "nbLines" : jsObj.length, "json" : jsObj };

		return vResult;
	}
	catch(e) {
		dout("Error parsing JSON ...",true,true);
		dout("RAW Out :\n" + result.stdout);
	// dout("stringified obj = " + JSON.stringify(jsObj));
	// dout("obj length = " + jsObj.length);
	// dout("obj 1 = " + JSON.stringify(jsObj[0]));
	// dout("obj 2 = " + JSON.stringify(jsObj[1]));

	}



}